"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const inquirer_1 = __importDefault(require("inquirer"));
const intl_relativetimeformat_1 = __importDefault(require("@formatjs/intl-relativetimeformat"));
require("@formatjs/intl-relativetimeformat/polyfill-locales");
const node_emoji_1 = require("node-emoji");
const chalk_1 = __importDefault(require("chalk"));
const chromatism = __importStar(require("chromatism"));
const Config_1 = require("../Config");
const graphql_1 = require("../utils/graphql");
const showSpinnerWhileProcessing_1 = require("../utils/showSpinnerWhileProcessing");
const getSavedQueries_1 = require("../utils/getSavedQueries");
const issuesQuery = fs_1.default.readFileSync(path_1.default.join(__dirname, "..", "queries", "issues.graphql"), "utf8");
exports.default = {
    command: "list <repo>",
    describe: "List issues for specified repository",
    builder: yargs => yargs.positional("repo", {
        describe: "owner and repository name (e.g. microsoft/TypeScript)",
    }),
    async handler(argv) {
        const config = Config_1.resolve(argv.config);
        const [owner, name] = argv.repo.split("/");
        if (!owner || !name) {
            throw new Error("Malformed. <repo> must be formatted with `owner/name`");
        }
        const graphql = graphql_1.getClient(config);
        const savedQueries = getSavedQueries_1.getSavedQueries(config, { owner, name });
        let query = savedQueries[0];
        if (savedQueries.length > 1) {
            const { queryIndex } = await inquirer_1.default.prompt([
                {
                    name: "queryIndex",
                    type: "rawlist",
                    message: `Multiple queries detected. Which one to use?`,
                    choices: savedQueries.map(({ name, query }, i) => ({
                        name: `${name} (${query})`,
                        value: i,
                    })),
                },
            ]);
            query = savedQueries[queryIndex];
        }
        const q = `is:open repo:${owner}/${name} ${query.query}`;
        const { search, } = await showSpinnerWhileProcessing_1.showSpinnerWhileProcessing(`Fetching issues ${owner}/${name} issues`, async () => graphql(issuesQuery, { q }));
        const issues = search.edges.map(({ node }) => node);
        const noAssociatedIssues = issues
            .filter((issue) => {
            const timelineItems = issue.timelineItems.edges
                .filter(Boolean)
                .map(({ node }) => node);
            const willBeClosedEvents = timelineItems.filter(({ willCloseTarget }) => willCloseTarget);
            return willBeClosedEvents.length === 0;
        })
            .sort((a, b) => new Date(a.updatedAt).getTime() - new Date(b.updatedAt).getTime());
        const now = Date.now();
        const formatter = new intl_relativetimeformat_1.default("en-US", { numeric: "auto" });
        noAssociatedIssues.forEach(({ number, title, url, updatedAt, comments, labels }) => {
            const relative = formatter.format(Math.round((new Date(updatedAt).getTime() - now) / 1000 / 60 / 60 / 24), "day");
            const numComments = comments.totalCount;
            const labelsStr = labels.nodes
                .map(({ name, color }) => chalk_1.default.bgHex(color).hex(chromatism.contrastRatio(`#${color}`).hex)(name))
                .join(", ");
            console.log(`\n#${number} ${chalk_1.default.bold(title)}`);
            console.log(node_emoji_1.emojify(`  :link: ${url}`));
            console.log(node_emoji_1.emojify(`  :label: ${labelsStr}`));
            console.log(node_emoji_1.emojify(`  :left_speech_bubble: ${numComments}, :timer_clock: ${relative}`));
        });
    },
};
